@@ -14,7 +14,7 @@
    You should have received a copy of the GNU General Public License
    along with Redshift.  If not, see <http://www.gnu.org/licenses/>.
 
-   Copyright (c) 2011  Jon Lund Steffensen <jonlst@gmail.com>
+   Copyright (c) 2013  Jon Lund Steffensen <jonlst@gmail.com>
 */
 
 #ifdef HAVE_CONFIG_H
@@ -51,10 +51,10 @@
 #define MAX(x,y)  ((x) > (y) ? (x) : (y))
 
 
-#if !(defined(ENABLE_RANDR) ||			\
-      defined(ENABLE_VIDMODE) ||		\
-      defined(ENABLE_WINGDI))
-# error "At least one of RANDR, VidMode or WinGDI must be enabled."
+#include "gamma-dummy.h"
+
+#ifdef ENABLE_DRM
+# include "gamma-drm.h"
 #endif
 
 #ifdef ENABLE_RANDR
@@ -72,10 +72,6 @@
 
 #include "location-manual.h"
 
-#ifdef ENABLE_GNOME_CLOCK
-# include "location-gnome-clock.h"
-#endif
-
 #ifdef ENABLE_GEOCLUE
 # include "location-geoclue.h"
 #endif
@@ -83,6 +79,9 @@
 
 /* Union of state data for gamma adjustment methods */
 typedef union {
+#ifdef ENABLE_DRM
+	drm_state_t drm;
+#endif
 #ifdef ENABLE_RANDR
 	randr_state_t randr;
 #endif
@@ -97,9 +96,21 @@ typedef union {
 
 /* Gamma adjustment method structs */
 static const gamma_method_t gamma_methods[] = {
+#ifdef ENABLE_DRM
+	{
+		"drm", 0,
+		(gamma_method_init_func *)drm_init,
+		(gamma_method_start_func *)drm_start,
+		(gamma_method_free_func *)drm_free,
+		(gamma_method_print_help_func *)drm_print_help,
+		(gamma_method_set_option_func *)drm_set_option,
+		(gamma_method_restore_func *)drm_restore,
+		(gamma_method_set_temperature_func *)drm_set_temperature
+	},
+#endif
 #ifdef ENABLE_RANDR
 	{
-		"randr",
+		"randr", 1,
 		(gamma_method_init_func *)randr_init,
 		(gamma_method_start_func *)randr_start,
 		(gamma_method_free_func *)randr_free,
@@ -111,7 +122,7 @@ static const gamma_method_t gamma_methods[] = {
 #endif
 #ifdef ENABLE_VIDMODE
 	{
-		"vidmode",
+		"vidmode", 1,
 		(gamma_method_init_func *)vidmode_init,
 		(gamma_method_start_func *)vidmode_start,
 		(gamma_method_free_func *)vidmode_free,
@@ -123,7 +134,7 @@ static const gamma_method_t gamma_methods[] = {
 #endif
 #ifdef ENABLE_WINGDI
 	{
-		"wingdi",
+		"wingdi", 1,
 		(gamma_method_init_func *)w32gdi_init,
 		(gamma_method_start_func *)w32gdi_start,
 		(gamma_method_free_func *)w32gdi_free,
@@ -133,6 +144,16 @@ static const gamma_method_t gamma_methods[] = {
 		(gamma_method_set_temperature_func *)w32gdi_set_temperature
 	},
 #endif
+	{
+		"dummy", 0,
+		(gamma_method_init_func *)gamma_dummy_init,
+		(gamma_method_start_func *)gamma_dummy_start,
+		(gamma_method_free_func *)gamma_dummy_free,
+		(gamma_method_print_help_func *)gamma_dummy_print_help,
+		(gamma_method_set_option_func *)gamma_dummy_set_option,
+		(gamma_method_restore_func *)gamma_dummy_restore,
+		(gamma_method_set_temperature_func *)gamma_dummy_set_temperature
+	},
 	{ NULL }
 };
 
@@ -140,9 +161,6 @@ static const gamma_method_t gamma_methods[] = {
 /* Union of state data for location providers */
 typedef union {
 	location_manual_state_t manual;
-#ifdef ENABLE_GNOME_CLOCK
-	location_gnome_clock_state_t gnome_clock;
-#endif
 #ifdef ENABLE_GEOCLUE
 	location_geoclue_state_t geoclue;
 #endif
@@ -164,20 +182,6 @@ static const location_provider_t location_providers[] = {
 		(location_provider_get_location_func *)
 		location_geoclue_get_location
 	},
-#endif
-#ifdef ENABLE_GNOME_CLOCK
-	{
-		"gnome-clock",
-		(location_provider_init_func *)location_gnome_clock_init,
-		(location_provider_start_func *)location_gnome_clock_start,
-		(location_provider_free_func *)location_gnome_clock_free,
-		(location_provider_print_help_func *)
-		location_gnome_clock_print_help,
-		(location_provider_set_option_func *)
-		location_gnome_clock_set_option,
-		(location_provider_get_location_func *)
-		location_gnome_clock_get_location
-	},
 #endif
 	{
 		"manual",
@@ -200,7 +204,7 @@ static const location_provider_t location_providers[] = {
 #define MIN_LON  -180.0
 #define MAX_LON   180.0
 #define MIN_TEMP   1000
-#define MAX_TEMP  10000
+#define MAX_TEMP  25000
 #define MIN_BRIGHTNESS  0.1
 #define MAX_BRIGHTNESS  1.0
 #define MIN_GAMMA   0.1
@@ -208,7 +212,7 @@ static const location_provider_t location_providers[] = {
 
 /* Default values for parameters. */
 #define DEFAULT_DAY_TEMP    5500
-#define DEFAULT_NIGHT_TEMP  3700
+#define DEFAULT_NIGHT_TEMP  3500
 #define DEFAULT_BRIGHTNESS   1.0
 #define DEFAULT_GAMMA        1.0
 
@@ -334,7 +338,7 @@ print_help(const char *program_name)
 		"  -o\t\tOne shot mode (do not continuously adjust"
 		" color temperature)\n"
 		"  -O TEMP\tOne shot manual mode (set color temperature)\n"
-		"  -p\t\tPrint mode (only print parameters and exit)"
+		"  -p\t\tPrint mode (only print parameters and exit)\n"
 		"  -x\t\tReset mode (remove adjustment from screen)\n"
 		"  -r\t\tDisable temperature transitions\n"
 		"  -t DAY:NIGHT\tColor temperature to set at daytime/night\n"),
@@ -433,17 +437,30 @@ provider_try_start(const location_provider_t *provider,
 	}
 
 	/* Set provider options from command line. */
+	const char *manual_keys[] = { "lat", "lon" };
+	int i = 0;
 	while (args != NULL) {
 		char *next_arg = strchr(args, ':');
 		if (next_arg != NULL) *(next_arg++) = '\0';
 
-		char *key = NULL;
+		const char *key = args;
 		char *value = strchr(args, '=');
-		if (value != NULL) {
-			key = args;
-			*(value++) = '\0';
+		if (value == NULL) {
+			/* The options for the "manual" method can be set
+			   without keys on the command line for convencience
+			   and for backwards compatability. We add the proper
+			   keys here before calling set_option(). */
+			if (strcmp(provider->name, "manual") == 0 &&
+			    i < sizeof(manual_keys)/sizeof(manual_keys[0])) {
+				key = manual_keys[i];
+				value = args;
+			} else {
+				fprintf(stderr, _("Failed to parse option `%s'.\n"),
+					args);
+				return -1;
+			}
 		} else {
-			value = args;
+			*(value++) = '\0';
 		}
 
 		r = provider->set_option(state, key, value);
@@ -458,6 +475,7 @@ provider_try_start(const location_provider_t *provider,
 		}
 
 		args = next_arg;
+		i += 1;
 	}
 
 	/* Start provider. */
@@ -515,13 +533,14 @@ method_try_start(const gamma_method_t *method,
 		char *next_arg = strchr(args, ':');
 		if (next_arg != NULL) *(next_arg++) = '\0';
 
-		char *key = NULL;
+		const char *key = args;
 		char *value = strchr(args, '=');
-		if (value != NULL) {
-			key = args;
-			*(value++) = '\0';
+		if (value == NULL) {
+			fprintf(stderr, _("Failed to parse option `%s'.\n"),
+				args);
+			return -1;
 		} else {
-			value = args;
+			*(value++) = '\0';
 		}
 
 		r = method->set_option(state, key, value);
@@ -920,9 +939,9 @@ main(int argc, char *argv[])
 	   try all providers until one that works is found. */
 	location_state_t location_state;
 
-	/* Location is not needed for reset mode
-	   or for manual temperature setting. */
-	if (mode != PROGRAM_MODE_RESET && mode != PROGRAM_MODE_MANUAL) {
+	/* Location is not needed for reset mode and manual mode. */
+	if (mode != PROGRAM_MODE_RESET &&
+	    mode != PROGRAM_MODE_MANUAL) {
 		if (provider != NULL) {
 			/* Use provider specified on command line. */
 			r = provider_try_start(provider, &location_state,
@@ -996,7 +1015,7 @@ main(int argc, char *argv[])
 		}
 
 		/* Color temperature at daytime */
-		if (temp_day < MIN_TEMP || temp_day >= MAX_TEMP) {
+		if (temp_day < MIN_TEMP || temp_day > MAX_TEMP) {
 			fprintf(stderr,
 				_("Temperature must be between %uK and %uK.\n"),
 				MIN_TEMP, MAX_TEMP);
@@ -1004,7 +1023,7 @@ main(int argc, char *argv[])
 		}
 	
 		/* Color temperature at night */
-		if (temp_night < MIN_TEMP || temp_night >= MAX_TEMP) {
+		if (temp_night < MIN_TEMP || temp_night > MAX_TEMP) {
 			fprintf(stderr,
 				_("Temperature must be between %uK and %uK.\n"),
 				MIN_TEMP, MAX_TEMP);
@@ -1014,7 +1033,7 @@ main(int argc, char *argv[])
 
 	if (mode == PROGRAM_MODE_MANUAL) {
 		/* Check color temperature to be set */
-		if (temp_set < MIN_TEMP || temp_set >= MAX_TEMP) {
+		if (temp_set < MIN_TEMP || temp_set > MAX_TEMP) {
 			fprintf(stderr,
 				_("Temperature must be between %uK and %uK.\n"),
 				MIN_TEMP, MAX_TEMP);
@@ -1056,34 +1075,41 @@ main(int argc, char *argv[])
 	   try all methods until one that works is found. */
 	gamma_state_t state;
 
-	if (method != NULL) {
-		/* Use method specified on command line. */
-		r = method_try_start(method, &state, &config_state,
-				     method_args);
-		if (r < 0) exit(EXIT_FAILURE);
-	} else {
-		/* Try all methods, use the first that works. */
-		for (int i = 0; gamma_methods[i].name != NULL; i++) {
-			const gamma_method_t *m = &gamma_methods[i];
-			r = method_try_start(m, &state, &config_state, NULL);
-			if (r < 0) {
-				fputs(_("Trying next method...\n"), stderr);
-				continue;
+	/* Gamma adjustment not needed for print mode */
+	if (mode != PROGRAM_MODE_PRINT) {
+		if (method != NULL) {
+			/* Use method specified on command line. */
+			r = method_try_start(method, &state, &config_state,
+					     method_args);
+			if (r < 0) exit(EXIT_FAILURE);
+		} else {
+			/* Try all methods, use the first that works. */
+			for (int i = 0; gamma_methods[i].name != NULL; i++) {
+				const gamma_method_t *m = &gamma_methods[i];
+				if (!m->autostart) continue;
+
+				r = method_try_start(m, &state, &config_state, NULL);
+				if (r < 0) {
+					fputs(_("Trying next method...\n"), stderr);
+					continue;
+				}
+
+				/* Found method that works. */
+				printf(_("Using method `%s'.\n"), m->name);
+				method = m;
+				break;
 			}
 
-			/* Found method that works. */
-			printf(_("Using method `%s'.\n"), m->name);
-			method = m;
-			break;
-		}
-
-		/* Failure if no methods were successful at this point. */
-		if (method == NULL) {
-			fputs(_("No more methods to try.\n"), stderr);
-			exit(EXIT_FAILURE);
+			/* Failure if no methods were successful at this point. */
+			if (method == NULL) {
+				fputs(_("No more methods to try.\n"), stderr);
+				exit(EXIT_FAILURE);
+			}
 		}
 	}
 
+	config_ini_free(&config_state);
+
 	switch (mode) {
 	case PROGRAM_MODE_ONE_SHOT:
 	case PROGRAM_MODE_PRINT:
@@ -1115,7 +1141,8 @@ main(int argc, char *argv[])
 			printf(_("Color temperature: %uK\n"), temp);
 			printf(_("Brightness: %.2f\n"), brightness);
 		}
-		if(mode == PROGRAM_MODE_PRINT) {
+
+		if (mode == PROGRAM_MODE_PRINT) {
 			exit(EXIT_SUCCESS);
 		}
 
